;********************************************************************
;  8051 Boot Loader ver 3.0
;  Download standard Intel hex format files
;  Assemble using ASM51.EXE by MetaLink Corp.
;  Author: Steven J. Merrifield, sjm@ee.latrobe.edu.au
;  sjm 11 Apr 95 Mike Sullivan
;  
;  Al Lipper 7/26/97 Implemented for ECU6
;********************************************************************

; Using the 87C51GB.
$MOD51GB

r0b0			equ	00h		; Register R0, Bank0
r1b0			equ	01h		; Register R1, Bank0
r2b0			equ	02h		; Register R2, Bank0
r3b0			equ	03h		; Register R3, Bank0
r4b0			equ	04h		; Register R4, Bank0

cret              equ 0dh             ; carriage return
lf                equ 0ah             ; linefeed

BaudLoad	equ	-39		;9600 at 12MHz
SMOD		equ	00000001b	; Bit to set in PCON for double baud rate


;********************************************************************
; Interrupt vector table
;********************************************************************
            org 0000h               ; System reset      RST
               	ljmp main
            org 000bh               ; Timer 0           TF0
            	ljmp wdt_isr      ;WDT ISR to keep CPU going

;********************************************************************
; Main program starts here
;********************************************************************
            org 0030h
main:           
                lcall wdt_init      ; init timer 0 for WDT reset
                lcall s_init          ; init serial port
                lcall intro         ; print welcome message
                lcall e_prompt      ; erase prompt
                LCALL F_ERASE       ; call flash erase routine
		    lcall hexload		; read hex file and save in flash
done_main:	    lcall end_msg		; Display done message
		    sjmp done_main	; loop forever		

; Read Intel hex file from serial port and store in flash
HEXLOAD:
        PUSH    PSW
                mov dptr,#p_rxhex	; Print 'Ready for hex file' message
                lcall outstr
ANL     PCON,#11110000B ; Clear flag bits

        SJMP    GetHex		; Wait for either ':' (Start),
                                ;  CR, or LF.
Ck_CR:  CJNE    A,#CRET,Ck_LF     ;  Any other chars will set the
        SJMP    WAIT            ;  error flag.

Ck_LF:  CJNE    A,#LF,Ck_C
        SJMP    WAIT

Ck_C:	CJNE    A,#3,CharErr	; Check for <Ctrl-C>
        SJMP    HEX_DONE

CharErr:ORL     PCON,#8         ; Set error flag
	CALL	C_OUT
        SJMP    WAIT

GetHex: JNB     TI,$
        CLR     TI
        MOV     SBUF,#'>'       ; Output pace character

WAIT:   CALL    GetChar         ; Wait to receive a character

        CJNE    A,#':',Ck_CR    ; Wait for start character ?
	CALL	C_OUT

        SETB    F0              ; --------- THE MAIN ENTRY POINT FOR HEXLOAD --------
        MOV     R2,#0           ; Start checksum
        CALL    RdByte          ; 1) Get record length
        MOV     R0,A            ; Save length

        CALL    RdByte          ; 2) Get high address
        MOV     DPH,A

        CALL    RdByte          ; 3) Get low address
        MOV     DPL,A

        CALL    RdByte          ; 4) Get record type
        JZ      GetDat          ; 0 if data, 1 if last record
        CLR     F0              ; Clear flag

GetDat: MOV     A,R0            ; Check record length
        JZ      IsEnd

DataIn: CALL    RdByte          ; 5) Get a byte
        JNB     F0,NoStor       ; Store if data record

WriteX: 
;	movx	@dptr, a
	  LCALL	WR_BYTE	  ; Write to flash
	  CJNE	A, #02h, NoHexErr	; Check for pgm error
	  MOV		R4, #02h		; Save code for program error
;	  SJMP	HEX_DONE		; Abort loading & report error		
NoHexErr:
	  INC		DPTR		  ; Point to next addr. in flash
	  
NoStor: DJNZ    R0,DataIn       ; Get more bytes
        DEC     R0              ; R0 = FF

IsEnd:  CALL    RdByte          ; Get checksum byte
        MOV     A,R2
        JZ      ChkOK
        ORL     PCON,#4
        
ChkOK:  MOV     A,#CRET
        CALL    C_OUT

        MOV     A,R0            ; Will be FF for data record
        JNZ     GetHex          ; Read another record?
                                
        MOV     A,#0CH          ; Now check for any errors
        ANL     A,PCON
        JZ      HEX_DONE

; Checksum Error        
cs_error:
	  mov 	dptr,#p_cserror
        lcall	outstr
	  mov		R4, #04		; Save code for Checksum error

HEX_DONE:
        POP     PSW
        RET                     ; All done so return

GetChar:JNB     RI,$            ; Character in SBUF ?
        MOV     A,SBUF
        CLR     RI
        CLR     ACC.7           ; Clear parity bit
	RET

; Get 2 chars & make a 2 digit byte.  Return byte in acc.
RdByte: SETB    PSW.1           

DoAgin: MOV     R3,A
        CALL    GetChar         ; Get a single character
        CALL    C_OUT           ; Echo it

        CLR     C
        SUBB    A,#'0'
        JNC     Do_1            ; Not a digit ?  (<30H ?)

        ORL     PCON,#8         ; Set error flag
        CLR     F0              ; DO NOT store this record

Do_1:   SUBB    A,#10           ; Test for A-F
        JC      IsNmbr
        SUBB    A,#7            ; Adjust for gap

IsNmbr: ADD     A,#10           ; Restore
        ANL     A,#0FH
        JBC     PSW.1,DoAgin    ; 2nd time ?

        SWAP    A
        ORL     A,R3
        SWAP    A               ; 2 digits in ACC

        XCH     A,R2            ; Get Checksum
        ADD     A,R2            ; Update it and
        XCH     A,R2            ;  get byte back.
        RET

C_OUT2: CALL    C_OUT           ; Send the character twice.

C_OUT:  JNB     TI,$            ; Wait til last character gone.
        CLR     TI              ; Clear interrupt flag.
        MOV     SBUF,A          ; Write out character.

        CJNE    A,#CRET,CONT      ; If it was <CR> then follow with <LF>.
        JNB     TI,$
        CLR     TI
        MOV     SBUF,#LF        ; Output linefeed.

CONT:   RET


;********************************************************************
;  Erase prompt message
;********************************************************************
e_prompt:       mov dptr,#p_ersprmpt   ; print erase prompt
                lcall outstr        
                lcall inchar         ; get char from serial port
                cjne a,#'e',e_prompt ; if "e" not pressed, loop
                lcall inchar                
                cjne a,#'r',e_prompt ; if "r" not pressed, loop
                lcall inchar                
                cjne a,#'a',e_prompt ; if "a" not pressed, loop
                lcall inchar                
                cjne a,#'s',e_prompt ; if "s" not pressed, loop
                lcall inchar                
                cjne a,#'e',e_prompt ; if "e" not pressed, loop
                ret

;********************************************************************
; Print welcome message
;********************************************************************
intro:          mov dptr,#p_intro1
                lcall outstr
                mov dptr,#p_intro2
                lcall outstr
                ret
;********************************************************************
; Print finished message or error
;********************************************************************
; NOTE: If more than one error occurs, only the first one will be reported
end_msg:        
		mov A, R4				; Get Error code
		cjne	A, #00h, erase_err	; Check for No Error

No_Err:	mov dptr,#p_finish
		lcall outstr
		sjmp	end_exit
	
erase_err:	cjne	A, #01h, pgm_err	; Check for Erase Error
		mov dptr,#p_ererror
		lcall outstr
		sjmp	end_exit

pgm_err:	cjne	A, #02h, zero_err; Check for Zeroing Error
		mov dptr,#p_pgmerror
		lcall outstr
		sjmp	end_exit

zero_err:	cjne	A, #03h, cksm_err; Check for Checksum Error
		mov dptr,#p_zrerror
		lcall outstr
		sjmp	end_exit

cksm_err:	cjne	A, #04h, inval_err; Check for undefined Error code
		mov dptr,#p_cserror
		lcall outstr
		sjmp	end_exit

inval_err:	mov dptr,#p_iverror
		lcall outstr
end_exit:	
		ret

;********************************************************************
; Wait until a char is received from the serial port, and return
; that char in the accumulator.
;********************************************************************
inchar:         jnb ri,inchar       ; wait until ri is set
                clr ri              ; clear interrupt
                mov a,sbuf          ; get character
                ret

;********************************************************************
; Wait until tx ready, then send a char out serial port
;********************************************************************
outchar:        jnb ti,outchar      ; wait until ti is set
                clr ti              ; clear it
                mov sbuf,a          ; send acc to serial buffer
                ret

;********************************************************************
; Send a null terminated string out serial port
;********************************************************************
outstr:
	clr a
                movc a,@a+dptr      ; get character
                jz exit             ; stop if char == null
                lcall outchar       ; else send it
                inc dptr            ; point to next char
                sjmp outstr
exit:           
	ret

;********************************************************************
; Init serial port
;********************************************************************
; *** Alternate baud rate init routine for 9600 baud at 12 Mhz (Using Timer2) ***
; Taken from Intel serio2m.asm.
;
S_INIT:
	CLR	TR1
	MOV	SCON,#01011010B	;TI set indicates transmitter ready.
				; mode 1,REN
	MOV	T2CON,#00110000B; Initialize Timer 2 as Baud Rate generator
	orl	PCON,#SMOD	; Set to double rate.
	setb	smod
	mov	rcap2h,#High BaudLoad	; Set reload value
	mov	rcap2l,#low Baudload
	setb	tr2		; start timer.
	ret
;

ck_error:       
                mov dptr,#p_cserror
                lcall outstr

stop:           sjmp stop

p_intro1:        db cret,'ECU Flash Memory Loader v1.1 7/24/97',cret,lf,0
p_intro2:        db cret,'(c) 1997 Injection Logic',cret,lf,0
p_ersprmpt:        db cret,lf,'Enter <erase> to Erase Chip',cret,lf,0
p_finish:        db cret,'Finished downloading: Switch Back to Run Mode',cret,lf,0
p_zero:        db 'Zeroing...',0
p_erase:        db 'Erasing...',0
p_erdone:       db 'Erase Completed',cret,lf,0
p_rxhex:       db cret,lf,'Ready to receive Intel Hex file',cret,lf,0

; Error message text.  Note R4 will contain error code (or 0 for no error)
p_ererror:        db cret,lf,'Flash Erase Error',0		; R4 = 1
p_pgmerror:        db cret,'Flash Programming Error during HEXLOAD',cret,lf,0	; R4 = 2	
p_zrerror:        db cret,lf,'Erase Zeroing Error',0	; R4 = 3
p_cserror:        db cret,'Serial Data Checksum Error Detected ',cret,lf,0	; R4 = 4
p_iverror:        db cret,'Invalid Error Code Returned ',cret,lf,0	; R4 > 4





;
;------------------------------------------------------------------------------
; Macros
;
;   The following macros are used to control assembly options.
;------------------------------------------------------------------------------

WAIT6us              MACRO                     ; wait 6 microseconds @ 12MHz
                push acc
        mov     acc,#01
                djnz    acc,$
                pop acc
      ENDM

;
; The 10us time delay
;

WAIT10us             MACRO                    ; wait 10 microseconds @ 12MHz
                push acc
                mov     acc,#03h
                djnz    acc,$
                pop acc
                      ENDM
;
; 10 ms time delay
;

WAIT10ms             MACRO                   ; wait 10 millisecond @ 12MHz
                push acc
                mov acc, #10
W10:   push acc
        mov     acc,#249
                djnz    acc,$
                mov     acc,#249
                djnz    acc,$
                pop acc
                djnz acc, W10
                pop acc
                    ENDM
;-----------------------------------------------------------------------------
;
; Critical Design Information:
;
;   1. For Flash devices, program/erase timing is extremely important. This
;      software uses a busy-wait loop to time the program and erase functions.
;      Processor/memory speed is therefore of critical importance to correct
;      and reliable programming and erasure. In general, use of this software
;      requires tailoring it to the timing of each individual system.
;   2. Programming Flash devices requires a 12.0V power supply.
;	   Control of Vpp is system dependent. Users must supply one function
;	   -- VPP -- that will be called to turn Vpp on or off.  On entry to VPP,
;	   the ACC register contains a code specifying the desired operation as
;	   follows:
;
;		ACC = 0 	Turn Vpp off
;		ACC = 1		Turn Vpp on
;      
;
;
; Abstract:
;
;   This software is designed to support the In-system programming requirements
;   of Intel's Flash memory.
;
;   This software module is supplied in source code form. The software is
;   written in MSC-51 assembly language using Intel ASM51 assembler. It
;   is designed to be assembled and linked with other system and application
;   software to form an integrated program. 
;
;   The following functions are supported:
;
;     1. Erase. Used to erase a single Flash memory before programming.
;     2. Program byte. Used to program single data elements, typically for 
;	 data collection applications.
;
; Function Details:
;
;   Overview:
;
;   Erase
;     This operation erases the Flash memory whose base address is
;     contained in the DPTR register.  The the base address of the 28F010
;     Flash memory is mapped at 0000h.
;
;   Write Byte
;     This operation writes a single byte (in ACC) into the Flash memory
;     at the address contained in the DPTR register:
;
;           ACC         Byte data
;           DPTR        Flash memory address
;
; Hardware Configurations:
;
;   This software is designed and implemented for the MCS-51 family.
;
; Language
;
;   This software is designed to assemble under the Intel ASM51 Assembler,
;   on the IBM PC and compatibles.
;==============================================================================
;------------------------------------------------------------------------------
; Miscellaneous Definitions
;------------------------------------------------------------------------------
M_ETRIES_LOW    equ 0E8H       	; 1000 erase tries maximum low byte
M_ETRIES_HIGH   equ 03H        	; 1000 erase tries maximum high byte
M_PTRIES        equ 25        	; 25 program tries maximum
F_BASE_ADDRESS	equ 0000H	; Flash base address
F_ENDL		equ 00H		; flash last address+1 (LSB) - FFFFH is last addr.
F_ENDH		equ 00H		; flash last address+1 (MSB)

;------------------------------------------------------------------------------
; Command Definitions
;
;   These commands are written to the Flash memory to run the internal state
;   machine.
;------------------------------------------------------------------------------

C_ERAS          equ     020H                    ; enter erase state
C_PROG          equ     040H                    ; enter program state
C_EVER          equ     0A0H                    ; enter erase verify state
C_PVER          equ     0C0H                    ; enter program verify state
C_READ          equ     000H                    ; enter read state



;------------------------------------------------------------------------------
; Error Definitions
;
;   These error values are used to identify MBM operation errors.
;------------------------------------------------------------------------------

OK              equ     0                       ; no error
E_ERASE         equ     1                       ; erase operation failed
E_PROGRAM       equ     2                       ; program operation failed



;--------------------------------------------------------------------------
; This routine is used to erase the content of a 28F010.
; 
;		Input parameters :
;			DPTR	  = base address of the 28F010
;		Oputput parameters :
;
;		Other registers used 
;			R2,R3	=	number of erase pulses (high byte in R3)
;			R4	=	Code indicating error 
;------------------------------------------------------------------------------


f_erase:	
            mov dptr,#p_zero		; print 'Zeroing...'
            lcall outstr
		push	dph			; save dptr
		push	dpl
		mov	r4, #00h		; Reset error code (0 = no error)
;
; Erase algorithm suggests testing to see if flash is erased before erasing it.  This
; step is skipped since flash should not ordinarily be previously erased at this point.

; First set all bytes to 00H in preparation for erase.
wr_next_page:
		mov	dptr,#f_base_address	; load the flash base address 
wr_next:
		mov	a,#00h
		call 	wr_byte				; programm 00h 
		cjne	a,#e_program,nexta

		mov	R4, #03h		; Save zeroing error code
		sjmp	erase_end		; return Zeroing error

nexta:
		inc	dptr			; next address
	;Check to see if current address has reached last flash address
		mov	a,dpl			; first check low byte
		cjne	a, #f_endl, wr_next
		mov	a,dph			; then check hi byte
		cjne	a, #f_endh, wr_next
;
;

;-------------- now erase the 28F010

erase:  
		mov dptr,#p_erase	; print 'Erasing...'
            lcall outstr
		mov	dptr, #f_base_address
		mov  	r2, #M_ETRIES_LOW	; max. erase retries low byte
		mov  	r3, #M_ETRIES_HIGH	; max. erase retries high byte
						; max. erase retries = 1000
 
era_again: 
		mov  	a, #c_eras 
		movx 	@dptr, a		; write E_code to CMD register 
		movx 	@dptr, a		; write again to confirm 
		wait10ms
		mov  	a, #c_ever 
		movx 	@dptr, a 		; stop, enter erase verify 
		wait6us       			; for high voltage switch 

; Verify that flash is erased (all bytes contain FFh)
loop:
		clr 	a
		movx 	a, @dptr		; read back 
		cjne 	a, #0ffh, next_pulse 
		inc  	dptr   			; next address 
	;Check to see if current address has reached last flash address
		mov	a,dpl			; first check low byte
		cjne	a, #f_endl, loop
		mov	a,dph			; then check hi byte
		cjne	a, #f_endh, loop

;erased:
		mov	a, #ok			; return code
		sjmp	erase_end

	; Erase attempt failed, try erase again if less than 1000 attempts
next_pulse:
		dec	r2				; increment erase count
		cjne r2, #0ffh, era_again	; if less than 1000 continue
		dec r3
		cjne r3, #0ffh, era_again
		mov	a, #e_erase			; more than 1000, return error code

erase_end:
		push	acc				; save return code
		mov	dptr,#f_base_address	; address to flash
		mov	a,#c_read		; write read code
		movx	@dptr,a
		pop	 acc			; restore return code
		pop	dpl
		pop dph

            cjne a, #e_erase, ck_pgmerr	; check for erase error
		mov dptr,#p_ererror	; print 'Erase Error'
            lcall outstr
		mov	r4, #01h		; Save erase error code
		sjmp	er_exit
		
; at this point, a programming error is due to flash not zeroing correctly
ck_pgmerr:      
            cjne a, #e_program, er_success	; check for zeroing error 
		mov dptr,#p_zrerror	; print 'Zeroing Error'
            lcall outstr
		mov	r0, #03h		; Save zeroing error code
		sjmp	er_exit

er_success:	
		mov dptr,#p_erdone	; print 'Erase Completed'
            lcall outstr
er_exit:		
		ret
 
;*************************************************************************
;
;  NAME:	WR_BYTE
;
;  ABSTRACT:	This routine program a byte of data of a Flash memory 
;	using Quick Pulse programming algorithm.  The address is loaded in
;	DPTR register and the data is loaded in the accummulator.
;
;  INPUTS:	DPTR	programmed address
;		A	data
;
;  OUTPUTS:	none
;
;  REGISTERS MODIFIED:
;		R0	save data
;		R1	number of programming loops
;
;  ERROR:	A = 00h	programmed without error
;			A = 02h	failed to program
;
;  SUBROUTINES ACCESS DIRECTLY:
;		wait10us	delay 10us	(macro)
;		wait6us		delay 6us	(macro)
;  
;*************************************************************************


wr_byte:	
		
		push	r0b0			; save r0
		push	r1b0			; save r1

		mov	r0,a			; save data in acc.
		mov	r1, #m_ptries		; allow 25 prog pulses
again:	
		mov	a,#c_prog		; load code to program
		movx	@dptr, a		; write prog code to CMD reg.
		mov	a,r0			; restore data
		movx	@dptr, a		; write data

		wait10us			; programming pulse width

		mov	a,#c_pver		; load program verify code
		movx	@dptr, a		; stop prog., enter verify mode
						; switch to program verify mode
		wait6us			; wait for 6us 
		clr a				; ACC=0 for next instruction
		movx	a,@dptr			; read back 
		clr	c
		subb	a, r0			; compare with programmed data 
		jz	wr_done			; next address
		djnz	r1,again		; after 25 loops
		mov	a,#e_program		; return with error code
		sjmp 	err_done
wr_done:	
		mov	a,#ok			; return ok code
err_done:
		push	acc			; store return code
		mov	a,#c_read		; write read code
		movx	@dptr,a
		pop 	acc			; restore return code
		
		pop	r1b0			; restore r1
		pop	r0b0			; restore r0
		ret

;WDT routines - see Intel appnotes (AB-44)
;Initialize timer 0 for watchdog ISR
wdt_init:
                setb ea
                setb et0

                mov tmod, #01h
                mov tl0, #7fh
                mov th0, #0c0h

                setb tr0
                ret

; WDT reset - WDT needs to be reset every 16ms or it will reset the CPU
; Timer 0 is used to call this ISR to do this reset.
wdt_isr:
      ; setb p1.5        ;debug
                clr     tr0

                mov     wdtrst, #1eh
                mov     wdtrst, #0e1h

                mov     tl0, #7fh
                mov     th0, #0c1h
                setb tr0
       ; clr p1.5       ;debug
		CPL 	P1.6	; Invert Reset IC's (MAX805L) Watchdog timer bit
                reti


end

